require 'DavinciRubyClientSDK/common'
require 'DavinciRubyClientSDK/blueprint'
require 'DavinciRubyClientSDK/receiver'
require 'DavinciRubyClientSDK/setting'
require 'DavinciRubyClientSDK/translator'
require 'DavinciRubyClientSDK/version'
require 'faraday'
include Faraday

module DavinciRubyClientSDK

  class API

    def initialize
      DavinciRubyClientSDK::Common.check_app_key
    end

    def draw(label, blueprints, options = {})
      options[:mode] ||= 'sync'
      @call_method = 'post'
      @call_path   = "/draw/#{options[:mode]}"
      @call_body   = {:label => label, :blueprint => blueprints}
      @call_body[:notify_url] = options[:notify_url] unless options[:notify_url].nil?
      self
    end

    def call
      begin

        url = DavinciRubyClientSDK::Setting.server_address + @call_path
        connect = Faraday.new(:url => url) do |faraday|
          faraday.request  :url_encoded
          faraday.adapter  Faraday.default_adapter
          faraday.basic_auth(DavinciRubyClientSDK::Setting.app_key, DavinciRubyClientSDK::Setting.app_secret)
        end

        response = connect.send("#{@call_method}") do |req|
          if @call_method    == 'post'
            req.body   = @call_body
          elsif @call_method == 'get'
            @call_params.map do |key, value|
              req.params[key] = value
            end
          end
        end.body

      rescue Exception => err
        @error = err

      ensure
        return @error, response

      end
    end

    def get(path, params)
      @call_method = 'get'
      @call_path   = path
      @call_params = params
      self
    end

    def post(path, body)
      @call_method = 'post'
      @call_path   = path
      @call_body   = body
      self
    end

  end

end
